package com.nisum.controller;



import com.nisum.bean.Item;
import com.nisum.repository.ItemReactiveRepository;
import com.nisum.service.ItemService;
import com.nisum.service.ItemServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;


@RestController
@Slf4j
public class ItemController {

    @Autowired
    ItemReactiveRepository itemReactiveRepository;

    @Autowired
    ItemService itemService;

    @PostMapping("/saveItem")
    @ResponseStatus(HttpStatus.CREATED)
    public Mono<Item> saveItem(@RequestBody Item item){

        return itemService.saveItem(item);


    }

    @GetMapping("/itemDetails"+"/{id}")
    public Mono<ResponseEntity<Item>> getOneItem(@PathVariable String id){

        return itemReactiveRepository.findById(id)
                .map((item) -> new ResponseEntity<>(item, HttpStatus.OK))
                .defaultIfEmpty(new ResponseEntity<>(HttpStatus.NOT_FOUND));

    }

    @GetMapping("/getItem"+"/{id}")
    public Mono<ResponseEntity<Item>> getItem(@PathVariable String id){

        return itemService.getItem(id)
                .map((item) -> new ResponseEntity<>(item, HttpStatus.OK))
                .defaultIfEmpty(new ResponseEntity<>(HttpStatus.NOT_FOUND));

    }


    @DeleteMapping("delete"+"/{id}")
    public Mono<Void> deleteItem(@PathVariable String id){

        return itemReactiveRepository.deleteById(id);


    }


    @PutMapping("/update"+"/{id}")
    public Mono<ResponseEntity<Item>> updateItem(@PathVariable String id,
                                                 @RequestBody Item item){

        return itemReactiveRepository.findById(id)
                .flatMap(currentItem -> {

                    currentItem.setPrice(item.getPrice());
                    currentItem.setDescription(item.getDescription());
                    return itemReactiveRepository.save(currentItem);
                })
                .map(updatedItem -> new ResponseEntity<>(updatedItem, HttpStatus.OK))
                .defaultIfEmpty(new ResponseEntity<>(HttpStatus.NOT_FOUND));

    }

    @PutMapping("/updateItem")
    public Mono<Item> updateItemDetails(@RequestBody Item item){

                   return itemService.updateItem(item);



    }

    @GetMapping("/getAllItems")
    public Flux<Item> getAllItems(){

        return itemService.getAllItems();

    }


}
