package com.nisum;


import com.nisum.bean.Item;
import com.nisum.repository.ItemReactiveRepository;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.AutoConfigureWebTestClient;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Arrays;
import java.util.List;

@SpringBootTest
@RunWith(SpringRunner.class)
@DirtiesContext
@AutoConfigureWebTestClient

public class ItemControllerTest {

    @Autowired
    WebTestClient webTestClient;

    @Autowired
    ItemReactiveRepository itemReactiveRepository;





    public List<Item> data() {

        return Arrays.asList(
                new Item(null, "Samsung TV", 399.99,"Super"),
                new Item("", "LG TV", 329.99,"Good"),
                new Item("2", "Apple Watch", 349.99,"Avg"),
                new Item("3", "Beats HeadPhones", 149.99,"Bad"));
    }
    @Before
    public void setUp(){
        itemReactiveRepository.deleteAll()
                .thenMany(Flux.fromIterable(data()))
                .flatMap(itemReactiveRepository::save)
                .doOnNext((item -> {
                    System.out.println("Inserted item is : " + item);
                }))
                .blockLast();
    }

    @Test
    public void getAllItemsTest(){
        webTestClient.get().uri("/getAllItems")
                .exchange()
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_VALUE)
                .expectBodyList(Item.class)
                .hasSize(4);

    }
    @Test
    public void saveItemTest(){

        Item item = new Item("123", "Iphone X", 999.99,"Good");

        webTestClient.post().uri("/saveItem")
                .contentType(MediaType.APPLICATION_JSON_UTF8)
                .body(Mono.just(item), Item.class)
                .exchange()
                .expectStatus().isCreated()
                .expectBody()
                .jsonPath("$.id").isNotEmpty()
                .jsonPath("$.name").isEqualTo("Iphone X")
                .jsonPath("$.price").isEqualTo(999.99)
                .jsonPath("$.description").isEqualTo("Good");


    }
    @Test
    public void updateItemTest(){

        double newPrice =129.99;
        Item item = new Item(null, "Apple Watch", newPrice,"Avg");

        webTestClient.put().uri( "/update"+"/{id}","2")
                .contentType(MediaType.APPLICATION_JSON_UTF8)
                .accept(MediaType.APPLICATION_JSON_UTF8)
                .body(Mono.just(item), Item.class)
                .exchange()
                .expectStatus().isOk()
                .expectBody()
                .jsonPath("$.price",newPrice);

    }

    @Test
    public void updateItem_notFoundTest(){
        double newPrice =129.99;
        Item item = new Item(null,"Beats HeadPhones", newPrice,"Good");

        webTestClient.put().uri("/update"+"/{id}","123")
                .contentType(MediaType.APPLICATION_JSON_UTF8)
                .accept(MediaType.APPLICATION_JSON_UTF8)
                .body(Mono.just(item), Item.class)
                .exchange()
                .expectStatus().isNotFound();

    }
    @Test
    public void deleteItemTest(){

        webTestClient.delete().uri("/delete"+"/{id})", "2")
                .accept(MediaType.APPLICATION_JSON_UTF8)
                .exchange()
                .expectStatus().isOk()
                .expectBody(Void.class);

    }
}
