package com.nisum;

import com.nisum.bean.Item;
import com.nisum.controller.ItemController;
import com.nisum.repository.ItemReactiveRepository;
import com.nisum.service.ItemService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.AutoConfigureWebTestClient;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import java.util.Arrays;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.refEq;
import static org.mockito.Mockito.when;

@SpringBootTest
@RunWith(SpringRunner.class)
@AutoConfigureWebTestClient
public class ItemDetailsIntegrationTest {



    @Autowired
    private WebTestClient webTestClient;

    @InjectMocks
    private ItemController itemController;

    @Mock
    private ItemService itemService;

    @Mock
    private ItemReactiveRepository itemReactiveRepository;
    public List<Item> data() {

        return Arrays.asList(
                new Item(null, "Samsung TV", 399.99,"Super"),
                new Item("", "LG TV", 329.99,"Good"),
                new Item("2", "Apple Watch", 349.99,"Avg"),
                new Item("3", "Beats HeadPhones", 149.99,"Bad"));
    }


    @Test
    public void save_UsingWebTestClient() {
        Item item=new Item("1","Beats HeadPhones", 129.33,"Good");
        webTestClient.post().uri("/saveItem").
                contentType(MediaType.APPLICATION_JSON).body(Mono.just(item), Item.class)
                .exchange()
                .expectStatus()
                .isCreated()
                .expectBody()
                .jsonPath("$.id")
                .isEqualTo("1");
    }

    @Test
    public void saveItemTest() {
        Item item=new Item("1","Beats HeadPhones", new Double(129.33),"Good");

        when(itemService.saveItem(item)).thenReturn(Mono.just(item));

        Mono<Item> itemMono = itemController.saveItem(item);

        StepVerifier.create(itemMono).expectNextMatches(responseDetails->{
            assertEquals("1", responseDetails.getId());
            assertEquals("Beats HeadPhones", responseDetails.getName());

            assertEquals(new Double(129.33), responseDetails.getPrice());
            assertEquals("Good", responseDetails.getDescription());
           return true;
        }).expectComplete().verify();
    }



    @Test
    public void getItemTest() {

        Item item=new Item("1","Beats HeadPhones", new Double(129.33),"Good");

        when(itemService.getItem("1")).thenReturn(Mono.just(item));

        Mono<ResponseEntity<Item>> itemMono = itemController.getItem("1");

        StepVerifier.create(itemMono).expectNextMatches(responseDetails->{
            assertEquals("1", responseDetails.getBody().getId());
            assertEquals("Beats HeadPhones", responseDetails.getBody().getName());

            assertEquals(new Double(129.33), responseDetails.getBody().getPrice());
            assertEquals("Good", responseDetails.getBody().getDescription());
            return true;
        }).expectComplete().verify();
    }

    @Test
    public void updateItemTest() {

        Item item=new Item("1","Beats HeadPhones", new Double(129.33),"Good");

        when(itemService.getItem("1")).thenReturn(Mono.just(item));

        Mono<ResponseEntity<Item>> itemMono = itemController.getItem("1");

        StepVerifier.create(itemMono).expectNextMatches(responseDetails->{
            assertEquals("1", responseDetails.getBody().getId());
            assertEquals("Beats HeadPhones", responseDetails.getBody().getName());

            assertEquals(new Double(129.33), responseDetails.getBody().getPrice());
            assertEquals("Good", responseDetails.getBody().getDescription());
            return true;
        }).expectComplete().verify();
    }

    @Test
       public void updateItemDetailsTest(){
        Item item=new Item("1","Beats HeadPhones", new Double(129.33),"Good");

        Item item2=new Item("1","Beats HeadPhones", new Double(129.33),"AVG");


        when(itemService.updateItem(item)).thenReturn(Mono.just(item2));

        Mono<Item> itemMono = itemController.updateItemDetails(item);
        StepVerifier.create(itemMono).expectNextMatches(responseDetails->{
            assertEquals("1", responseDetails.getId());
            assertEquals("Beats HeadPhones", responseDetails.getName());

            assertEquals(new Double(129.33), responseDetails.getPrice());
            assertEquals("AVG", responseDetails.getDescription());
            return true;
        }).expectComplete().verify();
    }

   @Test
    public void getAllItems(){

       List<Item> items = Arrays.asList(
               new Item(null, "Samsung TV", 399.99, "Super"),
               new Item("", "LG TV", 329.99, "Good"),
               new Item("2", "Apple Watch", 349.99, "Avg"),
               new Item("3", "Beats HeadPhones", 149.99, "Bad"));


       when(itemService.getAllItems()).thenReturn(Flux.fromIterable(items));

       Flux<Item> allItems = itemController.getAllItems();

       StepVerifier.create(allItems).expectNextCount(4).verifyComplete();


   }



}













